# -*- coding: utf-8 -*-
"""user object"""

import logging
import time

from dwho.classes.objects import DWhoObjectSQLBase

LOG = logging.getLogger('fd-vpn.objects.user')


class FdVpnObjectUser(DWhoObjectSQLBase):
    CONNECTION_NAME = 'db'
    TABLE_NAME      = 'user'
    COLUMNS         = ('id',
                       'username',
                       'email_address',
                       'status',
                       'roles',
                       'is_admin',
                       'created_at',
                       'updated_at')

    STATUS_ON       = 'ON'
    STATUS_OFF      = 'OFF'
    STATUS_DELETED  = 'DELETED'


    @staticmethod
    def get_status_list():
        return (FdVpnObjectUser.STATUS_OFF,
                FdVpnObjectUser.STATUS_ON,
                FdVpnObjectUser.STATUS_DELETED)

    def create(self, columns, values):
        self._validate_columns_values(columns, values)
        xdict       = dict(zip(columns, values))
        xdict['id'] = None

        if xdict.get('email_address'):
            xdict['email_address'] = xdict['email_address'].lower()

        if 'status' not in xdict:
            xdict['status'] = self.STATUS_ON

        xdict['created_at'] = time.strftime('%Y-%m-%d %H:%M:%S')
        xdict['updated_at'] = time.strftime('%Y-%m-%d %H:%M:%S')

        return DWhoObjectSQLBase.create(self, xdict.keys(), xdict.values())

    def update(self, columns, values, **conditions):
        self._validate_columns_values(columns, values)
        xdict               = dict(zip(columns, values))
        xdict['updated_at'] = time.strftime('%Y-%m-%d %H:%M:%S')

        if xdict.get('email_address'):
            xdict['email_address'] = xdict['email_address'].lower()

        return DWhoObjectSQLBase.update(self, xdict.keys(), xdict.values(), **conditions)
