# -*- coding: utf-8 -*-
"""vpn_client object"""

import logging
import time

from dwho.classes.objects import DWhoObjectSQLBase

LOG = logging.getLogger('fd-vpn.objects.vpn_client')


class FdVpnObjectVPNClient(DWhoObjectSQLBase):
    CONNECTION_NAME      = 'db'
    TABLE_NAME           = 'vpn_client'
    COLUMNS              = ('id',
                            'server_id',
                            'reference',
                            'last_octet',
                            'auth_method',
                            'auth_data',
                            'comment',
                            'enabled',
                            'connected',
                            'connected_at',
                            'disconnected_at',
                            'created_at',
                            'updated_at')

    AUTH_METHOD_NONE     = ''
    AUTH_METHOD_OTP      = 'otp'
    AUTH_METHOD_USERPASS = 'userpass'

    def __init__(self):
        from fd_vpn.objects.vpn_server import FdVpnObjectVPNServer
        from fd_vpn.objects.vpn_user import FdVpnObjectVPNUser

        self.obj_server = FdVpnObjectVPNServer
        self.obj_user   = FdVpnObjectVPNUser

        DWhoObjectSQLBase.__init__(self)

    @staticmethod
    def get_auth_method_list():
        return (FdVpnObjectVPNClient.AUTH_METHOD_NONE,
                FdVpnObjectVPNClient.AUTH_METHOD_OTP,
                FdVpnObjectVPNClient.AUTH_METHOD_USERPASS)

    def create(self, columns, values):
        self._validate_columns_values(columns, values)
        xdict       = dict(zip(columns, values))
        xdict['id'] = None

        if not xdict.get('auth_method'):
            xdict['auth_method'] = None
            xdict['auth_data'] = None

        xdict['created_at'] = time.strftime('%Y-%m-%d %H:%M:%S')
        xdict['updated_at'] = time.strftime('%Y-%m-%d %H:%M:%S')
        xdict['connected_at'] = None
        xdict['disconnected_at'] = None

        return DWhoObjectSQLBase.create(self, xdict.keys(), xdict.values())

    def update(self, columns, values, **conditions):
        self._validate_columns_values(columns, values)
        xdict               = dict(zip(columns, values))
        xdict['updated_at'] = time.strftime('%Y-%m-%d %H:%M:%S')

        return DWhoObjectSQLBase.update(self, xdict.keys(), xdict.values(), **conditions)

    def get_free_last_octets(self, server_id, client_id = None):
        if client_id:
            obj = self.get(**{'id': client_id})
        else:
            obj = None

        r       = list(range(self.config['vpn']['client']['last_octet']['min'],
                             self.config['vpn']['client']['last_octet']['max']))

        clients = self.get_all(**{'server_id': server_id})
        if not clients:
            return r

        for client in clients:
            if int(client['last_octet']) in r:
                r.remove(int(client['last_octet']))

        if obj:
            r.insert(0, obj['last_octet'])

        return r

    def get_all_server_user(self, to_dict = False, order = None, limit = None, offset = None, **conditions):
        db              = self.connect()

        table_server    = self.obj_server.TABLE_NAME
        table_user      = self.obj_user.TABLE_NAME

        columns         = (self.get_columns()
                           + self.obj_server.get_columns()
                           + self.obj_user.get_columns())

        query           = ("SELECT ${columns} " +
                           "FROM " + self.TABLE_NAME + " "
                           "INNER JOIN " + table_server + " "
                             "ON " + self.TABLE_NAME + ".server_id = " + table_server + ".id "
                           "LEFT JOIN " + table_user + " "
                             "ON " + table_server + ".user_id = " + table_user + ".id")

        cond_values     = None

        if conditions:
            cond            = self._prepare_condition(conditions)
            query          += " WHERE " + cond[0]
            cond_values     = cond[1]

        if order:
            query          += self._prepare_order(order)

        if limit:
            query          += self._prepare_limit(limit)

            if offset:
                query      += self._prepare_offset(offset)

        db['cursor'].query(query,
                           columns,
                           cond_values)
        res             = db['cursor'].fetchall()

        if not res:
            return False

        if not to_dict:
            return res

        r               = []

        for row in res:
            xobject = {}

            for column in columns:
                xobject[column] = self.db_prepare_column(row[column])

            r.append(xobject)

        return r

    """
    def get_server(self, to_dict = False, **conditions):
        db              = self.connect()

        table_server    = self.obj_server.TABLE_NAME
        columns         = self.get_columns() + self.obj_server.get_columns()

        query           = ("SELECT ${columns} " +
                           "FROM " + self.TABLE_NAME + " "
                           "INNER JOIN " + table_server + " "
                             "ON " + self.TABLE_NAME + ".server_id = " + table_server + ".id")

        cond_values     = None

        if conditions:
            cond            = self._prepare_condition(conditions)
            query          += " WHERE " + cond[0]
            cond_values     = cond[1]

        query          += " LIMIT 1"

        db['cursor'].query(query,
                           columns,
                           cond_values)
        row             = db['cursor'].fetchone()

        if not row:
            return False

        if not to_dict:
            return row

        r               = {}

        for column in columns:
            r[column] = self.db_prepare_column(row[column])

        return r
    """
