# -*- coding: utf-8 -*-
"""vpn_server object"""

import time
import logging

from dwho.classes.objects import DWhoObjectSQLBase

LOG = logging.getLogger('fd-vpn.objects.vpn_server')


class FdVpnObjectVPNServer(DWhoObjectSQLBase):
    CONNECTION_NAME = 'db'
    TABLE_NAME      = 'vpn_server'
    COLUMNS         = ('id',
                       'user_id',
                       'reference',
                       'cidr',
                       'port_number',
                       'enabled',
                       'created_at',
                       'updated_at')

    def __init__(self):
        from fd_vpn.objects.vpn_user import FdVpnObjectVPNUser

        self.obj_user = FdVpnObjectVPNUser

        DWhoObjectSQLBase.__init__(self)

    def create(self, columns, values):
        self._validate_columns_values(columns, values)
        xdict       = dict(zip(columns, values))
        xdict['id'] = None

        xdict['created_at'] = time.strftime('%Y-%m-%d %H:%M:%S')
        xdict['updated_at'] = time.strftime('%Y-%m-%d %H:%M:%S')

        return DWhoObjectSQLBase.create(self, xdict.keys(), xdict.values())

    def update(self, columns, values, **conditions):
        self._validate_columns_values(columns, values)
        xdict               = dict(zip(columns, values))
        xdict['updated_at'] = time.strftime('%Y-%m-%d %H:%M:%S')

        return DWhoObjectSQLBase.update(self, xdict.keys(), xdict.values(), **conditions)

    def get_all_user(self, to_dict = False, order = None, limit = None, offset = None, with_user = False, **conditions):
        db              = self.connect()

        table_user      = self.obj_user.TABLE_NAME

        columns         = (self.get_columns()
                           + self.obj_user.get_columns())

        if not with_user:
            join_type = 'LEFT'
        else:
            join_type = 'INNER'

        query           = ("SELECT ${columns} " +
                           "FROM " + self.TABLE_NAME + " "
                           + join_type + " JOIN " + table_user + " "
                             "ON " + self.TABLE_NAME + ".user_id = " + table_user + ".id")

        cond_values     = None

        if conditions:
            cond            = self._prepare_condition(conditions)
            query          += " WHERE " + cond[0]
            cond_values     = cond[1]

        if order:
            query          += self._prepare_order(order)

        if limit:
            query          += self._prepare_limit(limit)

            if offset:
                query      += self._prepare_offset(offset)

        db['cursor'].query(query,
                           columns,
                           cond_values)
        res             = db['cursor'].fetchall()

        if not res:
            return False

        if not to_dict:
            return res

        r               = []

        for row in res:
            xobject = {}

            for column in columns:
                xobject[column] = self.db_prepare_column(row[column], column)

            r.append(xobject)

        return r

    """
    def get_by_user(self, to_dict = False, **conditions):
        db              = self.connect()

        table_server    = self.TABLE_NAME
        table_user      = self.obj_user.TABLE_NAME
        columns_user    = self.obj_user.get_columns()

        query           = ("SELECT ${columns} " +
                           "FROM " + table_server + " "
                           "INNER JOIN " + table_user + " "
                             "ON " + table_server + ".user_id = " + table_user + ".id")

        cond_values     = None

        if conditions:
            cond            = self._prepare_condition(conditions)
            query          += " WHERE " + cond[0]
            cond_values     = cond[1]

        query          += " LIMIT 1"

        db['cursor'].query(query,
                           columns_user,
                           cond_values)
        row             = db['cursor'].fetchone()

        if not row:
            return False

        if not to_dict:
            return row

        r               = {}

        for column in columns_user:
            r[column] = self.db_prepare_column(row[column])

        return r
    """
