# -*- coding: utf-8 -*-
"""credentials plugin"""

import json
import logging

from fd_vpn.objects.user import FdVpnObjectUser

from dwho.classes.plugins import DWhoPluginSQLBase, PLUGINS

LOG         = logging.getLogger('fd-vpn.plugins.credentials')
ROLE_DEVOPS = 'devops'

class FdVpnCredentialsPlugin(DWhoPluginSQLBase):
    PLUGIN_NAME = 'credentials'

    def __init__(self):
        self.objects = {'user': FdVpnObjectUser()}
        self.request = None

        DWhoPluginSQLBase.__init__(self)

    def init(self, config):
        DWhoPluginSQLBase.init(self, config)

        self.objects['user'].init(config)

    def at_start(self):
        return

    def set_request(self, request):
        self.request = request

    def get_info(self, request = None):
        if not request:
            request = self.request

        user = (request._SERVER.get('HTTP_AUTH_USER')
                or request.get_headers().get('x-auth-user-id'))
        if not user:
            return False

        return self.objects['user'].get(**{'username': user})

    def is_admin(self, request = None):
        if not request:
            request = self.request

        user = self.get_info(request)

        if not user:
            return False

        return bool(int(user['is_admin']))

    def get_roles(self, request = None):
        if not request:
            request = self.request

        user = self.get_info(request)

        if not user or not user['roles']:
            return []

        return json.loads(user['roles'])

    def has_role(self, role, request = None):
        if not request:
            request = self.request

        if self.is_admin(request):
            return True

        roles = self.get_roles(request)

        if not roles:
            return False

        return role in roles


if __name__ != "__main__":
    def _start():
        PLUGINS.register(FdVpnCredentialsPlugin())
    _start()
